"""
====================
 PyS60RemoteControl
====================
http://pys60rc.sourceforge.net/
(c) 2007 Dorian Scholz
Released under the GNU General Public Licence
"""
from RemoteControlPlugin import RemoteControlPlugin
from WindowHandler import WindowHandler

WM_APPCOMMAND = 0x319

APPCOMMAND = {}

APPCOMMAND['BROWSER_BACKWARD'] = 1
APPCOMMAND['BROWSER_FORWARD'] = 2
APPCOMMAND['BROWSER_REFRESH'] = 3
APPCOMMAND['BROWSER_STOP'] = 4
APPCOMMAND['BROWSER_SEARCH'] = 5
APPCOMMAND['BROWSER_FAVORITES'] = 6
APPCOMMAND['BROWSER_HOME'] = 7
APPCOMMAND['VOLUME_MUTE'] = 8
APPCOMMAND['VOLUME_DOWN'] = 9
APPCOMMAND['VOLUME_UP'] = 10
APPCOMMAND['MEDIA_NEXTTRACK'] = 11
APPCOMMAND['MEDIA_PREVIOUSTRACK'] = 12
APPCOMMAND['MEDIA_STOP'] = 13
APPCOMMAND['MEDIA_PLAY_PAUSE'] = 14
APPCOMMAND['LAUNCH_MAIL'] = 15
APPCOMMAND['LAUNCH_MEDIA_SELECT'] = 16
APPCOMMAND['LAUNCH_APP1'] = 17
APPCOMMAND['LAUNCH_APP2'] = 18
APPCOMMAND['BASS_DOWN'] = 19
APPCOMMAND['BASS_BOOST'] = 20
APPCOMMAND['BASS_UP'] = 21
APPCOMMAND['TREBLE_DOWN'] = 22
APPCOMMAND['TREBLE_UP'] = 23
APPCOMMAND['MICROPHONE_VOLUME_MUTE'] = 24
APPCOMMAND['MICROPHONE_VOLUME_DOWN'] = 25
APPCOMMAND['MICROPHONE_VOLUME_UP'] = 26
APPCOMMAND['HELP'] = 27
APPCOMMAND['FIND'] = 28
APPCOMMAND['NEW'] = 29
APPCOMMAND['OPEN'] = 30
APPCOMMAND['CLOSE'] = 31
APPCOMMAND['SAVE'] = 32
APPCOMMAND['PRINT'] = 33
APPCOMMAND['UNDO'] = 34
APPCOMMAND['REDO'] = 35
APPCOMMAND['COPY'] = 36
APPCOMMAND['CUT'] = 37
APPCOMMAND['PASTE'] = 38
APPCOMMAND['REPLY_TO_MAIL'] = 39
APPCOMMAND['FORWARD_MAIL'] = 40
APPCOMMAND['SEND_MAIL'] = 41
APPCOMMAND['SPELL_CHECK'] = 42
APPCOMMAND['DICTATE_OR_COMMAND_CONTROL_TOGGLE'] = 43
APPCOMMAND['MIC_ON_OFF_TOGGLE'] = 44
APPCOMMAND['CORRECTION_LIST'] = 45
APPCOMMAND['MEDIA_PLAY'] = 46
APPCOMMAND['MEDIA_PAUSE'] = 47
APPCOMMAND['MEDIA_RECORD'] = 48
APPCOMMAND['MEDIA_FAST_FORWARD'] = 49
APPCOMMAND['MEDIA_REWIND'] = 50
APPCOMMAND['MEDIA_CHANNEL_UP'] = 51
APPCOMMAND['MEDIA_CHANNEL_DOWN'] = 52
APPCOMMAND['DELETE'] = 53
APPCOMMAND['DWM_FLIP3D'] = 54


class AppCommandControl(RemoteControlPlugin):

  def __init__(self, rc, timer_callback = None, interval = 0.5):
    RemoteControlPlugin.__init__(self, rc, self.timer_callback, interval)
    self.window_handler = WindowHandler(rc.log, self.conf.get('window_class', default = None), self.conf.get('window_name', default = None))
    self.status_fields = self.conf.get('status_fields', default = {})

  def command_callback(self, command, key_event):
    self.rc.log(3, command)
    try:
      self.window_handler.window.SendMessage(WM_APPCOMMAND, self.window_handler.window.GetSafeHwnd(), APPCOMMAND[command['appcommand']] * 0x10000)
    except:
      if self.window_handler.get_window():
        self.window_handler.window.SendMessage(WM_APPCOMMAND, self.window_handler.window.GetSafeHwnd(), APPCOMMAND[command['appcommand']] * 0x10000)
    
  def start(self):
    self.last_status_text = ''
    RemoteControlPlugin.start(self)
    
  def timer_callback(self):
    status_text = None
    try:
      status_text = self.get_status_text()
    except:
      if self.window_handler.get_window():
        status_text = self.get_status_text()
    if status_text != None and self.last_status_text != status_text:
      self.show_text(status_text)
      self.last_status_text = status_text

  def get_status_text(self):
    status = {}
    for field_name, dialog_ids in self.status_fields.items():
      dialog = self.window_handler.window
      for dialog_id in dialog_ids[:-1]:
        dialog = dialog.GetDlgItem(dialog_id)
      status[field_name] = dialog.GetDlgItemText(dialog_ids[-1])
    return '\n'.join(status.values())
    