import threading

class TimerRepeat(threading._Timer):
  """
    This is a wrapper class around threading.Timer to contiously call a function every interval, until the cancel method is called.
  """
  def __init__(self, intervall, function, args = [], kwargs = {}):
    threading._Timer.__init__(self, intervall, function, args, kwargs)
    self.setDaemon(True)

  def run(self):
    while not self.finished.isSet():
      self.function(*self.args, **self.kwargs)
      self.finished.wait(self.interval)


if __name__ == '__main__':    
  import time
  
  def callback(interval, id):
    print '%2.1fs in callback(%d)' % (time.clock(), id),
    time.sleep(interval)
    print '%2.1fs out callback(%d)' % (time.clock(), id)
  
  tr = TimerRepeat(0.3, callback, (0.3, 1))
  print '%2.1fs tr.start()\n' % time.clock()
  tr.start()
  time.sleep(3)
  tr.cancel()
  print '\n\n%2.1fs tr.cancel()' % time.clock()
  
  time.sleep(1)
  
  tr = TimerRepeat(0.6, callback, (0.1, 2))
  print '\n\n%2.1fs tr.start()\n' % time.clock()
  tr.start()
  time.sleep(3)
  tr.cancel()
  print '\n\n%2.1fs tr.cancel()' % time.clock()

  raw_input('press enter to close...')
