"""
====================
 PyS60RemoteControl
====================
http://pys60rc.sourceforge.net/
(c) 2007 Dorian Scholz
Released under the GNU General Public Licence

Based on 'phcomm.py' by Kari Pulli.
"""

import os
import sys
import binascii
import marshal
import thread
import socket

def connect_phone2PC(config_file_path, interactive = True):
  addr = discover_address(config_file_path, interactive)
  if addr:
    sock = socket.socket(socket.AF_BT, socket.SOCK_STREAM)
    sock.connect(addr)
#    socket.set_security(sock, socket.AUTH)
    return sock
  else:
    print 'Failed to connect.'
    return None

def discover_address(config_file_path, interactive = True):
  """ discover_address(config_file, interactive = True)
  called on the phone side
  config_file_path stores the address and port of previous connection
  if interactive == False that address is used, otherwise
  the user is prompted to select device and service
  """
  import appuifw

  try:        
    config = eval(open(config_file_path, 'r').read())
  except:
    config = {}

  address = config.get('target', '')

  if address and not interactive:
    return address

  if address:
    choice = appuifw.popup_menu([u'Default host', u'Other...'], u'Connect to:')
    if choice == 0:
      return address
    if choice == None:
      return None # popup menu was cancelled

  # address not stored, or want a new host
  print "Discovering..."
  addr, services = socket.bt_discover()
  print "Discovered: %s, %s" % (addr, services)
  if len(services) > 1:
    choices = services.keys()
    choices.sort()
    service_list = [str.rstrip('\x00').decode() for str in choices]
    choice  = appuifw.popup_menu(service_list, u'Choose port:')
    if choice == None:
      print 'No service choosen...'
      return None
    port = services[choices[choice]]
  else:
    port = services.values()[0]
  address = (addr, port)
  config['target'] = address
  # make sure the configuration file exists
  if not os.path.isdir(os.path.dirname(config_file_path)):
      os.makedirs(os.path.dirname(config_file_path))
  # store the configuration file
  open(config_file_path, 'wt').write(repr(config))
  return address


class socketfile:
  """ gives a socket a file-like api """
  def __init__(self, sock):
    self.read_buf = ''
    self.sock = sock
    self.log = lambda *args, **kwargs: False

  def readline(self, size = None, eol = '\n'):
    line = ''
    while line[-1:] != eol:
      line += self.read_until(eol, max_read = 1)
    self.log(3, 'readline: return: "' + line + '"')
    return line
      
  def read_until(self, delimiter, max_read = 2048):
    self.log(5, 'read_until: read_buf: "' + self.read_buf + '"')
    parts = self.read_buf.split(delimiter, 1)
    if len(parts) == 2:
      result = parts[0] + delimiter
    else:
      self.read_buf += self.sock.recv(max_read)
      parts = self.read_buf.split(delimiter, 1)
      result = parts[0]
      if len(parts) == 2:
        result += delimiter
    self.read_buf = self.read_buf[len(result):]
    return result
  
  def read(self, max_read = 2048):
    self.log(4, 'read: read_buf: "' + self.read_buf + '"')
    if len(self.read_buf) < max_read:
      self.read_buf += self.sock.recv(max_read - len(self.read_buf))
    result = self.read_buf[:max_read]
    self.read_buf = self.read_buf[len(result):]
    return result
  
  def write(self, msg):
    self.sock.send(msg)

  def flush(self):
    pass

  def close(self):
    if self.sock:
      try:
        # workaround for the bug where select on phone keeps a pointer in C++ when at timeout
        self.sock._sock.close()
      except:
        pass
      self.sock.close()
      self.sock = None


class SvrCli:
  """ base class for Server and Client"""

  MAX_SEND = 8192
  MAX_READ_FROM_FILE = 10240

  def __init__(self, stream, verbose = 0):
    self.stream = stream
    self.verbose = verbose
    self.logout = sys.stdout
    self.write_lock = thread.allocate_lock()
    self.log(4, '__init__')

  def log(self, level, *args):
    if level <= self.verbose:
      print >> self.logout, repr(args)
      return True
    return False
    
  def recv_data(self): 
    datainfo_string = self.stream.readline()
    datainfo = eval(datainfo_string)
    self.log(4, "Content-Length: %d\n" % datainfo['content-length'])
    received_bytes = 0
    data = ''
    while received_bytes < datainfo['content-length']:
      data_chunk = self.stream.read(min(datainfo['content-length'] - received_bytes, SvrCli.MAX_SEND))
      received_bytes += len(data_chunk)
      self.log(5, 'Received: %d bytes (%3.1f%%)' % (received_bytes, (100.0 * received_bytes / datainfo['content-length'])))
      data += data_chunk
    self.log(3, 'Received: %d bytes.' % received_bytes)
    if datainfo.has_key('crc'):
      crc = binascii.crc32(data)
      if datainfo['crc'] != crc:
        str = """\
        expected crc %d, calculated crc %d
        expected content length %d, content length %d
        """ % (datainfo['crc'], crc, datainfo['content-length'], len(data))
        raise IOError('CRC error while receiving data:\n' + str)
    return data
    
  def recv_file(self, filename):
    # the reason for this function is to store everything into a file and not have
    # the whole content in the memory at the same time
    datainfo_string = self.stream.readline()
    datainfo = eval(datainfo_string)
    self.log(4, 'Content-Length: %d' % datainfo['content-length'])
    received_bytes = 0
    data = ''
    if datainfo.has_key('crc'):
      crc = 0
    else:
      crc = None
    self.log(4, 'Receiving data...')
    f = open(filename, 'wb')
    while received_bytes < datainfo['content-length']:
      data_chunk = self.stream.read(min(datainfo['content-length'] - received_bytes, SvrCli.MAX_SEND))
      received_bytes += len(data_chunk)
      f.write(data_chunk)
      if crc != None:
        crc = binascii.crc32(data_chunk, crc)
    f.close()
    self.log(3, 'Received: %d bytes.' % received_bytes)
    if crc != None and datainfo['crc'] != crc:
      str = """\
      expected crc %d, calculated crc %d
      expected content length %d, content length %d
      """ % (crc32, crc, datainfo['content-length'], received_bytes)
      raise IOError("CRC error while receiving data:\n"+str)

  def send_file(self, filename, crc_check = False):
    self.log(3, 'send_file: ' + filename)
    # Send the data in little bits because the Bluetooth serial connection may lose data on large sends.
    # Also want to conserve RAM consumption. First, read file in chunks to get data length and crc
    content_length = os.stat(filename).st_size
    header = {'content-length' : content_length}
    f = open(filename, 'rb')
    if crc_check:
      crc = 0
      data_chunk = f.read(SvrCli.MAX_READ_FROM_FILE)
      while data_chunk:
        crc = binascii.crc32(data_chunk, crc)
        data_chunk = f.read(SvrCli.MAX_READ_FROM_FILE)
      # rewind the file
      f.seek(0)
      header['crc'] = crc
    self.send_line(repr(header))
    sent_bytes = 0
    while sent_bytes < content_length:
      n = min(content_length - sent_bytes, SvrCli.MAX_SEND)
      self.stream.write(f.read(n))
      self.stream.flush()
      sent_bytes += n
      self.log(5, "Sent: %d bytes (%3.1f%%)\r" % (sent_bytes, (100.0 * sent_bytes / content_length)))
    f.close()
    
  def send_data(self, data, crc_check = False):
    self.log(5, 'send_data')
    header = {'content-length' : len(data)}
    if crc_check:
      header['crc'] = binascii.crc32(data)
    self.send_line(repr(header))
    sent_bytes = 0    
    # Send the data in little bits because the Bluetooth serial connection may lose data on large sends.
    while sent_bytes < len(data):
      n = min(len(data) - sent_bytes, SvrCli.MAX_SEND)
      self.stream.write(data[sent_bytes : sent_bytes + n])
      self.stream.flush()
      sent_bytes += n
      self.log(5, 'Sent: %d bytes (%3.1f%%)' % (sent_bytes, (100.0 * sent_bytes / len(data))))

  def send_pyobj(self, obj):
    """ send a python object, e.g., a unicode string """
#    data = marshal.dumps(obj)
    data = repr(obj)
    self.log(3, 'send_pyobj: %s' % data)
    self.send_data(data)
    
  def recv_pyobj(self):
    """ receive a python object, e.g., a unicode string """
    data = self.recv_data()
    self.log(3, 'recv_pyobj: %s' % data)
#    obj = marshal.loads(data)
    obj = eval(data)
    return obj
    
  def send_line(self, cmd):
    """ send a line """
    if cmd[-1] != '\n': 
      cmd += '\n'
    self.log(3, 'send_line: ' + cmd)
    self.stream.write(cmd)
    self.stream.flush()

  def send(self, data_type, data, crc_check = False):
    if data_type == 'line':
      self.send_line(data)
    elif data_type == 'file':
      self.send_file(data, crc_check)
    elif data_type == 'data':
      self.send_data(data, crc_check)
    elif data_type == 'object':
      self.send_pyobj(data)
    
  def send_atomic(self, list, wait_flag = True):
    if self.write_lock.acquire(wait_flag):
      for send_entry in list:
        self.send(send_entry[0], send_entry[1])
      self.write_lock.release()

  def read(self, *args, **kwargs):
    return self.stream.read(*args, **kwargs)

  def readline(self, *args, **kwargs):
    return self.stream.readline(*args, **kwargs)

  def write(self, *args, **kwargs):
    return self.stream.write(*args, **kwargs)

  def flush(self, *args, **kwargs):
    return self.stream.flush(*args, **kwargs)

  def close(self, *args, **kwargs):
    return self.stream.close(*args, **kwargs)

if __name__ == '__main__':
  sc = SvrCli(None)
  sc.log(0, 'working...')
