"""
====================
 PyS60RemoteControl
====================
http://pys60rc.sourceforge.net/
(c) 2007 Dorian Scholz
Released under the GNU General Public Licence
"""
ALWAYS_ASK_FOR_BLUETOOTH_DEVICE = False

import sys, appuifw, graphics, e32, sysinfo
from types import IntType
sys.path.append('c:\\Python\\libs')
sys.path.append('e:\\Python\\libs')
sys.path.append('libs')
import pys60rc_com

class ClientGui:

  def __init__(self, server):
    self._server = server

    appuifw.app.title = u'PyS60RemoteControl'
    appuifw.app.screen = 'full'
#    appuifw.app.exit_key_handler = self.OnExit # does not seem to work somehow, but we can use the key otherwise anyhow
    appuifw.app.exit_key_handler = lambda: False # deactivate right softkey handler, so we can use the key for the remote control
    
    self._canvas = appuifw.Canvas(redraw_callback = self.OnCanvasUpdate, event_callback = self.OnCanvasEvent)
    self._dblbuf = graphics.Image.new(sysinfo.display_pixels())
    self._background = graphics.Image.new(sysinfo.display_pixels())

    # use compatibility code for text drawing on 2nd edition
    if e32.s60_version_info[0] < 3:
      self._dblbuf._text_old = self._dblbuf.text
      self._dblbuf.text = self._text_2nd

    self.set_font()
    self.drawing_queue = []

    appuifw.app.body = self._canvas

    self.set_menu()
    
    self._dblbuf.clear()
    self.OnCanvasUpdate(None)


  def OnMenu(self, menu_cmd):
    self._server.send_line(menu_cmd)

  def OnCanvasEvent(self, event):
#    self._server.send_atomic([
#      ('line', 'remote_key'),
#      ('object', event),
#    ])
    self._server.send_line('remote_key %s' % repr(event))

  def OnCanvasUpdate(self, rect):
    if(self._canvas and self._dblbuf):
      self._canvas.blit(self._dblbuf)
  
  def OnExit(self):
    self._server.quit()

  def _parseMenuList(self, entries):
    menu = ()
    for (menu_text, menu_cmd) in entries:
      if type(menu_cmd) == type(''):
        menu += ((unicode(menu_text), lambda self=self, menu_cmd=menu_cmd: self.OnMenu(menu_cmd)),)
      else:
        menu += ((unicode(menu_text), self._parseMenuList(menu_cmd)),)
    return menu

  def set_menu(self, entries = []):
    appuifw.app.menu = list(self._parseMenuList(entries))
    appuifw.app.menu.append((u'Exit', self.OnExit))


  default_font = {
    'size' : 6,
    'font' : 'normal',
    'bold' : -1,
    'italic' : -1,
    'fill' : 0x000000,
  }
  def set_font(self, **kwargs):
    self.font = {}
    for key in ClientGui.default_font.keys():
      if kwargs.has_key(key):
        self.font[key] = kwargs[key]
      else:
        self.font[key] = ClientGui.default_font[key]


  text_align = {
    'left' : 0,
    'right' : -1,
    'center' : -0.5,
    'top' : 0.8,
    'bottom' : -0.2,
    'middle' : 0.3,
  }
  def draw_text(self, text_position_list):
    defaults = ('left', 'middle')
    for entry in text_position_list:
      text, x, y, h_align, v_align = entry + defaults[len(entry) - 3:]
      h_align_factor = ClientGui.text_align[h_align]
      v_align_factor = ClientGui.text_align[v_align]
      lines = text.decode('latin_1').split('\n')
      line_height = self.font['size'] * 2.5
      y += line_height * v_align_factor * len(lines)
      for line in lines:
        line_width = self._dblbuf.text((0, 0), line, dodraw = False, getwidth = True, **self.font)
        self._dblbuf.text((x + int(line_width * h_align_factor), int(y)), line, **self.font)
        y += line_height

  font_widths = {
    'title' : 12,
    'normal' : 8,
    'dense' : 5,
    'symbol' : 5,
    'legend' : 5,
    'annontation' : 5,
  }
  def _text_2nd(self, coords, text, **kwargs):
    if not kwargs.has_key('dodraw') or kwargs.get('dodraw'):
      kwargs_2nd = {}
      font = kwargs.get('font')
      if font:
        kwargs_2nd['font'] = font
      fill = kwargs.get('fill')
      if fill:
        kwargs_2nd['fill'] = fill
      self._dblbuf._text_old(coords, text, **kwargs_2nd)
    if kwargs.get('getwidth'):
      font_width = ClientGui.font_widths.get(kwargs.get('font'))
      if not font_width:
        font_width = 8
      return len(text) * font_width

  def draw_image(self, filepath_or_hash, x = 0, y = 0, bg = True):
    if type(filepath_or_hash) == IntType:
      filepath = self._server.cache.get(filepath_or_hash)
      if not filepath:
        print 'hash-value not found: %d' % filepath_or_hash
        return
    else:
      filepath = filepath_or_hash
    image = graphics.Image.open(filepath)
    if bg:
      self._background.blit(image, target = (x, y))
    else:
      self._dblbuf.blit(image, target = (x, y))


  def restore_bg(self, x = 0, y = 0, w = 0, h = 0):
    if w == 0 or h == 0:
      bottom_right = self._background.size
    else:
      bottom_right = (x + w, y + h)
    self._dblbuf.blit(self._background, target = (x, y), source = ((x, y), bottom_right))


  def draw(self):
    for i in range(len(self.drawing_queue)):
      entry = self.drawing_queue.pop(0)
      getattr(self, entry[0])(*entry[1], **entry[2])
    self.OnCanvasUpdate(None)



class SocketStdio:
    def __init__(self, server):
        self.server = server
        self.writebuf = []
        self.readbuf  = []
        
    def read(self, n=1):
        if not self.readbuf:
            self.readbuf = self.readline()
        readchars, self.readbuf = self.readbuf[0:n], self.readbuf[n:]
        return readchars

    def write(self, str):
        if str:
            self.writebuf.append(str)
        if '\n' in self.writebuf:
            self.flush()

    def flush(self):
        if self.writebuf:
            #self.server.send_atomic([('line', 'output'), ('object', self.writebuf)])
            self.server.send_line('output %s' % repr(self.writebuf))
            self.writebuf = []



class Cache:
  def __init__(self, path):
    self.path = path.rstrip('\\') + '\\'
    self.cache_file = path + 'cache.dat'
    self.hash2name = {}
    self.name2hash = {}
    self.load()
    
  def load(self):
    try:
      f = open(self.cache_file, 'rb')
      self.hash2name, self.name2hash = eval(f.read())
      f.close()
    except IOError:
      return False
    return True
    
  def save(self):
    try:
      f = open(self.cache_file, 'wb')
      f.write(repr((self.hash2name, self.name2hash)))
      f.close()
    except IOError:
      return False
    return True
  
  def add(self, filehash, filename):
    # remove old hash value for this filename
    old_filehash = self.name2hash.get(filename)
    if old_filehash:
      del self.hash2name[old_filehash]
    filepath = self.path + filename
    self.hash2name[filehash] = filepath # add new hash value for this filename
    self.name2hash[filename] = filehash # assosiate this filename with the new filehash
    return filepath

  def get(self, filehash):
    return self.hash2name.get(filehash)



class BTserver(pys60rc_com.SvrCli):

  def run(self):
    self.stdin  = sys.stdin
    self.stdout = sys.stdout
    self.stderr = sys.stderr
    sys.stdout = SocketStdio(self)
    sys.stderr = SocketStdio(self)
    sys.stdin  = SocketStdio(self)

    try:
      self.download_queue = []
      self.cache = Cache('d:\\')
      self.gui = ClientGui(self)
      x, y = sysinfo.display_pixels()
      self.gui.draw_text([('PyS60RemoteControl\nwaiting for connection...', int(x / 2), int(y / 2), 'center', 'middle')])
      self.gui.OnCanvasUpdate(None)
      self.finished = False
      self.send_init()
      while not self.finished:
        line = self.readline().rstrip()
        if line:
          self.log(4, 'line: "' + line + '"')
          self.evaluate(line)
        e32.ao_yield() # give the gui some time
    except:
      sys.stdout = self.stdout
      sys.stderr = self.stderr
      sys.stdin  = self.stdin
      import traceback
      traceback.print_exc()
    else:
      sys.stdout = self.stdout
      sys.stderr = self.stderr
      sys.stdin  = self.stdin
    self.close()
    
  def quit(self):
    self.log(1, 'quit!')
    self.send_line('bye')
    self.finished = True
    
  def send_init(self):
      self.send_line('init %d %d' % sysinfo.display_pixels())
    
  def evaluate(self, line):
    self.log(3, 'eval: "' + line + '"')
    # parse the command
    parts = line.split(' ')
    head = parts[0]
    if head == 'init': # init\n
      self.send_init()
    elif head == 'quit': # quit\n
      self.quit()
    elif head == 'bye': # bye\n
      pass
    elif head == 'set_menu': # setmenu\n{pyobj(menuentries)}
      menuentries = self.recv_pyobj()
      self.gui.set_menu(menuentries)
    elif head == 'draw': # draw\n{drawing_queue}
      drawing_queue = self.recv_pyobj()
      download_queue = []
      for entry in drawing_queue:
        if entry[0] == 'draw_image' and type(entry[1][0]) == IntType:
          filehash = entry[1][0]
          if not self.cache.get(filehash):
            download_queue.append(filehash)
      self.gui.drawing_queue += drawing_queue
      if download_queue:
        self.download_queue += download_queue
        self.send_line('send_files ' + repr(download_queue))
      elif not self.download_queue:
        self.gui.draw()
    elif head == 'draw_image': # draw_image filename [x y]\n{image}
      filepath = 'd:\\' + parts[1]
      self.recv_file(filepath)
      try:
        x, y = parts[2:4]
        self.gui.draw_image(filepath, int(x), int(y), bg = False)
        self.gui.OnCanvasUpdate(None)
      except IndexError:
        self.gui.draw_image(filepath, bg = False)
        self.gui.OnCanvasUpdate(None)
      except ValueError:
        print 'ValueError: %s' % repr(parts[2:4])
    elif head == 'file': # file filename filehash\n{file}
      filename, filehash_string = parts[1:3]
      filehash = int(filehash_string)
      self.recv_file(self.cache.add(filehash, filename))
      self.cache.save()
      if filehash in self.download_queue:
        self.download_queue.remove(filehash)
        if self.download_queue == []:
          self.gui.draw()
    else:
      print 'Unknown command: ' + line
    


def main(interactive = True):
  print 'remote.py: start...'
  server_sock = pys60rc_com.connect_phone2PC('c:\\System\\Apps\\Python\\pys60rc_com.cfg', interactive = interactive)
  if server_sock:
    try:
      BTserver(pys60rc_com.socketfile(server_sock), verbose = 0).run()
    except:
      server_sock.close()
      print 'remote.py: run failed'
      import traceback
      traceback.print_exc()
  else:
    print 'remote.py: could not connect'
  print 'remote.py: done.'


if __name__ == '__main__':
  main(ALWAYS_ASK_FOR_BLUETOOTH_DEVICE) # False => non interactive connect
