"""
====================
 PyS60RemoteControl
====================
http://pys60rc.sourceforge.net/
(c) 2007 Dorian Scholz
Released under the GNU General Public Licence
"""
from RemoteControlPlugin import RemoteControlPlugin

class KeymapLearner(RemoteControlPlugin):

  def __init__(self, rc):
    RemoteControlPlugin.__init__(self, rc)

  def start(self):
    RemoteControlPlugin.start(self)
    # copy key_list from global config
    self.key_list = [value for value in self.rc.conf.get('key_list', default = [])]
    if len(self.key_list) > 0:
      self.key_event = self.keymap_learner
      self.keymap_learner(None)
    else:
      self.rc.log(0, 'No key_list! Please check the main conf!')
  
  def keymap_learner(self, key_event):
    if key_event:
      if key_event['type'] != RemoteControlPlugin.key_event_types['down']: # only use key-down events
        return
      self.scancode2key[key_event['scancode']] = self.key_list.pop(0)
    if self.key_list:
      key = self.key_list[0]
      self.rc.log(2, 'Learning Keymap: Press key: ' + key)
      x, y = self.rc.skin.key_coords[key]
      text_position_list = [
        (key, x, y, 'center', 'middle'),
      ]
      text_position_list2 = [
        (key, x + 1, y + 1, 'center', 'middle'),
      ]
      self.rc.restore_bg(*self.rc.skin.keypad_rect)
      self.rc.set_font(**self.rc.skin.keypad_font)
      self.rc.draw_text(text_position_list)
      self.rc.set_font(**self.rc.skin.keypad_font2)
      self.rc.draw_text(text_position_list2)
      self.show_text('Learning Keymap\nPress "%s" key' % key)
    else:
      self.rc.conf.set('scancode2key', self.scancode2key)
      self.key_event = self.key_dispatcher
      self.rc.restore_bg(*self.rc.skin.keypad_rect)
      self.show_text('Finished learning the Keymap.\nYou can now use the plugins.')
    return False # never dispatch the keyevent
