"""
====================
 PyS60RemoteControl
====================
http://pys60rc.sourceforge.net/
(c) 2007 Dorian Scholz
Released under the GNU General Public Licence
"""
# inspired by the python winamp control by Shalabh Chaturvedi

from RemoteControlPlugin import RemoteControlPlugin
import os, re, wx, win32gui, win32api, time

WM_COMMAND = 0x0111
WM_USER    = 0x400

class Winamp(RemoteControlPlugin):

  def __init__(self, rc):
    RemoteControlPlugin.__init__(self, rc, self.timer_callback, 0.4)
    self.last_exec_time = 0
    # load values from plugin config
    self.exec_path = self.conf.get('exec_path', default = 'c:/Programs/winamp/winamp.exe')

  def start(self):
    self.rc.log(2, 'looking for winamp')
    self.findWinamp()
    self.last_status_text = ''
    RemoteControlPlugin.start(self)

  def timer_callback(self):
    if not self.findWinamp():
      return
    status = self.getPlayingStatus()
    artist, track = self.getCurrentTrackName()
    track_length, track_time = self.getTrackStatus()
    time_remaining = track_length - track_time
    time_m = time_remaining / 60000
    time_s = (time_remaining % 60000) / 1000
    track_number = self.getCurrentTrack() + 1
    num_tracks = self.getNumTracks()
    status_text = '%s Track %d of %d : -%d:%02d\n%s\n%s' % (status, track_number, num_tracks, time_m, time_s, artist, track)
    if self.last_status_text != status_text:
      self.show_text(status_text)
      self.last_status_text = status_text

  def command_callback(self, command, key_event = None):
    if not self.findWinamp():
      return
    self.rc.log(3, command)
    return win32api.SendMessage(self.hWinamp, WM_COMMAND, command['id'], 0)
    
  def toogle_playback(self, command, key_event):
    "starts playing if paused or stoped, pauses if playing"
    if not self.findWinamp():
      return
    self.rc.log(3, command)
    if self.getPlayingStatus() == 'playing':
      self.command_callback({'id': command['id_pause']})
    else:
      self.command_callback({'id': command['id_play']})
    
  def findWinamp(self):
    """
    Searches for a window with the window class Winamp v1.x and stores it's handle.
    If non is found, it spawns a new process launching 'self.exec_path'.
    """
    try:
      self.hWinamp = win32gui.FindWindow('Winamp v1.x', None)
    except:
      if time.time() - self.last_exec_time > 10: # give winamp at most 10 seconds to start up
        print 'winamp not running! executing winamp...'
        self.last_exec_time = time.time()
        os.spawnl(os.P_NOWAIT, self.exec_path)
      return False
    return True

  def usercommand(self, id, data=0):
    return win32api.SendMessage(self.hWinamp, WM_USER, data, id)
    
  def getVersion(self):
    "returns the version number of winamp"
    iVersionNumber = self.usercommand(0)
    sVersionString = hex(iVersionNumber)
    sVersionString = sVersionString[2:3] + '.' + sVersionString[3:]
    self.sVersion = sVersionString
    return self.sVersion

  def getPlayingStatus(self):
    "returns the current status string which is one of 'playing', 'paused' or 'stopped'"
    iStatus = self.usercommand(104)
    if iStatus == 1:
      return 'playing'
    elif iStatus == 3:
      return 'paused'
    else:
      return 'stopped'
      
  def getTrackStatus(self):
    "returns a tuple (total_length, current_position) where both are in msecs"
    iTotalLength = self.usercommand(105, 1) * 1000 # the usercommand returns the number in seconds
    iCurrentPos  = self.usercommand(105, 0)
    return (iTotalLength, iCurrentPos)

  def setCurrentTrack(self, iTrackNumber):
    "changes the track selection to the number specified"    
    return self.usercommand(121, iTrackNumber)

  def getCurrentTrack(self):
    return self.usercommand(125)

  def getCurrentTrackName(self):
    title = win32gui.GetWindowText(self.hWinamp)
    track = re.match('^(\d+\. )?(.+?)( - Winamp.*)?$', title).group(2)
    matches = re.match('^(.+?) - (.+?)$', track)
    if matches:
      return matches.groups()
    return track, ''

  def seekWithinTrack(self, iPositionMsecs):
    "seeks within currently playing track to specified milliseconds since start"
    return self.usercommand(106, iPositionMsecs)
    
  def getNumTracks(self):
    "returns number of tracks in current playlist"
    return self.usercommand(124)

  def dumpList(self):
    "dumps the current playlist into WINAMPDIR/winamp.m3u"
    return self.usercommand(120)
